/* File:  InetConnection.hpp 

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the InetConnection class/library

The Initial Developer of the Original Code is Gregory Riggs.
Portions created by Gregory Riggs are Copyright (C) 2000, 2001
Gregory Riggs.  All Rights Reserved.

Contributor(s):
  Gregory Riggs <griggs@itqlata.com> <greg@nmia.com>
  Jerry Karasz <jkarasz@itqlata.com>
  ITQ LATA, LLC. <wctpxml@itqlata.com>

*/

#ifndef INET_CONNECTION_HPP
#define INET_CONNECTION_HPP

#include <netdb.h>
#include <sys/socket.h>
#include <iostream.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <fcntl.h>
#include <signal.h>

#include "XString.hpp"

#define INET_CONN_OK 0
#define INET_CONN_OKAY 0
#define INET_CONN_ERROR 1
#define INET_CONN_CLOSE 2
#define INET_CONN_TIMEOUT 3
#define INET_CONN_CANCEL 4

#define BUFFER_SIZE 2048

class InetConnection
{
public:

    InetConnection();
    virtual ~InetConnection();

    int make_nonblocking(XString& error);
    int put_fd(XString& error, int fd, const struct sockaddr_in& addr);
    int open_conn(XString &error, const XString& hostname, int port);
    int close_conn(XString& error);
    bool is_closed(void);
    void fast_close(void);

    int ready_for_io(XString& error, bool read_flag, bool write_flag);
    int ready_for_read(XString& error);
    int ready_for_write(XString& error);

    int writen(XString& error, const char *data, int length);
    int readn(XString& error, XString &data, int length);
    int read_available(XString& error, XString& data);

    static int lookup_ipaddr(XString& error, const XString& hostname,
        struct sockaddr_in *addr);
    static XString translate_error(int errnum);
    static XString translate_host_error(int herrnum);

    struct sockaddr_in client_addr;
    int sock_fd;
    time_t session_start;
    time_t session_limit;
    int polling_interval;
    char buffer[BUFFER_SIZE];
    bool cancel_flag;
    static bool abort_flag;
};

#endif
