/* File:  wctpXml.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_HPP
#define WCTP_XML_HPP

#include "XString.hpp"
#include "TStack.hpp"
#include "strings.h"

enum wctp_parse_t {
    WCTP_PARSE_OKAY   = 0,
    WCTP_PARSE_ERROR  = 1
};

enum wctp_bool_t {
    WCTP_BOOL_FALSE   = 0,
    WCTP_BOOL_TRUE    = 1,
    WCTP_BOOL_DEFAULT = 2
};

enum wctp_encoding_t {
    WCTP_ENCODING_DEFAULT  = 0,
    WCTP_ENCODING_STANDARD = 1,
    WCTP_ENCODING_BASE64   = 2,
    WCTP_ENCODING_UUENCODE = 3
};

enum wctp_format_t {
    WCTP_FORMAT_ALPHANUMERIC     = 0,
    WCTP_FORMAT_TRANSPARENT_DATA = 1,
    WCTP_FORMAT_ALL              = 2,
    WCTP_FORMAT_DEFAULT          = 3
};

enum wctp_notification_t {
    WCTP_NOTIFICATION_DEFAULT   = 0,
    WCTP_NOTIFICATION_QUEUED    = 1,
    WCTP_NOTIFICATION_DELIVERED = 2,
    WCTP_NOTIFICATION_READ      = 3
};

enum wctp_priority_t {
    WCTP_PRIORITY_DEFAULT  = 0,
    WCTP_PRIORITY_LOW      = 1,
    WCTP_PRIORITY_NORMAL   = 2,
    WCTP_PRIORITY_HIGH     = 3
};

enum wctp_transparent_t {
    WCTP_TRANSPARENT_DEFAULT   = 0,
    WCTP_TRANSPARENT_OPAQUE    = 1,
    WCTP_TRANSPARENT_FLEXSUITE = 2
};

enum wctp_xml_t {
    WCTP_XML_UNDEFINED               = 0,
    WCTP_XML_ALPHANUMERIC            = 2,
    WCTP_XML_CHOICE                  = 6,
    WCTP_XML_CLIENT_MESSAGE          = 7,
    WCTP_XML_CLIENT_MESSAGE_CONTROL  = 8,
    WCTP_XML_CLIENT_MESSAGE_REPLY    = 9,
    WCTP_XML_CLIENT_ORIGINATOR       = 10,
    WCTP_XML_CLIENT_QUERY            = 11,
    WCTP_XML_CLIENT_QUERY_RESPONSE   = 12,
    WCTP_XML_CLIENT_RESPONSE_HEADER  = 13,
    WCTP_XML_CLIENT_STATUS_INFO      = 14,
    WCTP_XML_CLIENT_SUCCESS          = 15,
    WCTP_XML_CONFIRMATION            = 16,
    WCTP_XML_FAILURE                 = 20,
    WCTP_XML_LOOKUP_DATA             = 21,
    WCTP_XML_LOOKUP_MESSAGE_CONTROL  = 22,
    WCTP_XML_LOOKUP_RESPONSE         = 23,
    WCTP_XML_LOOKUP_SUBSCRIBER       = 24,
    WCTP_XML_MCR                     = 25,
    WCTP_XML_MESSAGE                 = 26,
    WCTP_XML_MESSAGE_CONTROL         = 27,
    WCTP_XML_MESSAGE_RECEIVED        = 30,
    WCTP_XML_MESSAGE_REPLY           = 31,
    WCTP_XML_MESSAGE_TEXT            = 33,
    WCTP_XML_NO_MESSAGES             = 36,
    WCTP_XML_NOTIFICATION            = 37,
    WCTP_XML_OPERATION               = 38,
    WCTP_XML_ORIGINATOR              = 39,
    WCTP_XML_PAYLOAD                 = 41,
    WCTP_XML_POLL_FOR_MESSAGES       = 42,
    WCTP_XML_POLL_RESPONSE           = 43,
    WCTP_XML_RECIPIENT               = 48,
    WCTP_XML_RESPONSE_HEADER         = 49,
    WCTP_XML_STATUS_INFO             = 50,
    WCTP_XML_SUBMIT_CLIENT_HEADER    = 52,
    WCTP_XML_SUBMIT_CLIENT_MESSAGE   = 53,
    WCTP_XML_SUBMIT_CLIENT_RESPONSE  = 54,
    WCTP_XML_SUBMIT_HEADER           = 55,
    WCTP_XML_SUBMIT_REQUEST          = 56,
    WCTP_XML_SUCCESS                 = 58,
    WCTP_XML_TRANSPARENT_DATA        = 59,
    WCTP_XML_VERSION_QUERY           = 83,
    WCTP_XML_VERSION_QUERY_RESPONSE  = 84,
    WCTP_XML_VERSION_CONTACT_INFO    = 85,
    WCTP_XML_VERSION_DTD_SUPPORT     = 86
};

class wctpXml 
{
public:

    //////////////////////////////////
    // returns current timestamp (not thread-safe)
    static const char *
    GenerateTimestamp(void);

    //////////////////////////////////
    // returns a new message id (not thread-safe)
    static const char *
    GenerateMessageID(void);

    //////////////////////////////////
    // wctp_bool_t operations      
    static const char*
    bool_to_str(wctp_bool_t);
    static wctp_bool_t
    str_to_bool(const char *); 
        
    //////////////////////////////////
    // wctp_encoding_t operations      
    static const char*
    encoding_to_str(wctp_encoding_t);
    static wctp_encoding_t
    str_to_encoding(const char *); 
        
    //////////////////////////////////
    // wctp_format_t operations      
    static const char*
    format_to_str(wctp_format_t);
    static wctp_format_t
    str_to_format(const char *); 
        
    //////////////////////////////////
    // wctp_notification_t operations      
    static const char*
    notification_to_str(wctp_notification_t);
    static wctp_notification_t
    str_to_notification(const char *); 
        
    //////////////////////////////////
    // wctp_priority_t operations      
    static const char*
    priority_to_str(wctp_priority_t);
    static wctp_priority_t
    str_to_priority(const char *); 
        
    //////////////////////////////////
    // wctp_transparent_t operations      
    static const char*
    transparent_to_str(wctp_transparent_t);
    static wctp_transparent_t
    str_to_transparent(const char *); 
};

#endif
