/* File:  wctpXmlClientStatusInfo.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_CLIENT_STATUS_INFO_HPP
#define WCTP_XML_CLIENT_STATUS_INFO_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlClientResponseHeader.hpp"
#include "wctpXmlFailure.hpp"
#include "wctpXmlNotification.hpp"

class wctpXmlClientStatusInfo : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlClientStatusInfo(); 
    wctpXmlClientStatusInfo(const wctpXmlClientStatusInfo &); 
    ~wctpXmlClientStatusInfo(); 
    wctpXmlClientStatusInfo& operator=(const wctpXmlClientStatusInfo &);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // ClientStatusInfo Type
    wctp_xml_t 
    GetClientStatusInfoType(void);

    ////////////////////////////
    // ClientResponseHeader Child Element (always holds)
    wctpXmlClientResponseHeader* 
    GetClientResponseHeader(void);

    ////////////////////////////
    // Failure Child Element (holds only one of set)
    wctpXmlFailure* 
    GetFailure(void);
    void // specify to hold this item from the set
    HoldFailure(void);

    ////////////////////////////
    // Notification Child Element (holds only one of set)
    wctpXmlNotification* 
    GetNotification(void);
    void // specify to hold this item from the set
    HoldNotification(void);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlClientStatusInfo &);

    virtual void 
    Free(void);

    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctp_xml_t                   m_ClientStatusInfoType;
    wctpXmlClientResponseHeader *m_pClientResponseHeader;
    wctpXmlFailure              *m_pFailure;
    wctpXmlNotification         *m_pNotification;
   
};
#endif
