/* File:  wctpXmlLookupResponse.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_LOOKUP_RESPONSE_HPP
#define WCTP_XML_LOOKUP_RESPONSE_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlOriginator.hpp"
#include "wctpXmlRecipient.hpp"
#include "wctpXmlLookupData.hpp"
#include "wctpXmlFailure.hpp"

class wctpXmlLookupResponse : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlLookupResponse(); 
    wctpXmlLookupResponse(const wctpXmlLookupResponse &); 
    ~wctpXmlLookupResponse(); 
    wctpXmlLookupResponse& operator=(const wctpXmlLookupResponse &);

    ////////////////////////////
    // reset object to default state 
    virtual void 
    Clear(void);

    virtual XString 
    AssembleOperation(XmlContext& ctx);

    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);

    ////////////////////////////
    // LookupResponse Type
    wctp_xml_t 
    GetLookupResponseType(void);

    ////////////////////////////
    // Originator Child Element (always holds)
    wctpXmlOriginator* 
    GetOriginator(void);

    ////////////////////////////
    // Recipient Child Element (always holds)
    wctpXmlRecipient* 
    GetRecipient(void);

    ////////////////////////////
    // LookupData Child Element (holds only one of set)
    wctpXmlLookupData* 
    GetLookupData(void);
    void // specify to hold this item from the set
    HoldLookupData(void);

    ////////////////////////////
    // Failure Child Element (holds only one of set)
    wctpXmlFailure* 
    GetFailure(void);
    void // specify to hold this item from the set
    HoldFailure(void);

    ////////////////////////////
    // responseToMessageID attribute
    const char * 
    GetResponseToMessageID(void) const;
    void  
    SetResponseToMessageID(const char *);

    ////////////////////////////
    // transactionID attribute
    const char * 
    GetTransactionID(void) const;
    void  
    SetTransactionID(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlLookupResponse &);

    virtual void 
    Free(void);
        
    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);
    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctp_xml_t         m_LookupResponseType;
    wctpXmlOriginator *m_pOriginator;
    wctpXmlRecipient  *m_pRecipient;
    wctpXmlLookupData *m_pLookupData;
    wctpXmlFailure    *m_pFailure;
    XString            m_responseToMessageID;
    XString            m_transactionID;

};
#endif
