/* File:  wctpXmlMCR.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_MCR_HPP
#define WCTP_XML_MCR_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlMessageText.hpp"
#include "wctpXmlChoice.hpp"

class wctpXmlMCR : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlMCR(); 
    wctpXmlMCR(const wctpXmlMCR &); 
    ~wctpXmlMCR(); 
    wctpXmlMCR& operator=(const wctpXmlMCR &);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject (XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // MessageText Child Element (always holds)
    wctpXmlMessageText * 
    GetMessageText(void);

    ////////////////////////////
    // Choice Child Elements (one or more)

    // erase all Choice elements
    void 
    ClearChoice(void);  
    // get Choice element at given position
    // ---- pos is a zero-based index 
    wctpXmlChoice* 
    GetChoice(int pos); 
    // add Choice element at end of array
    wctpXmlChoice* 
    AddChoice(void);
    // erase Choice element at given position
    // ---- pos is a zero-based index 
    void  
    EraseChoice(int pos);
    // return number of Choice elements
    int 
    NumChoice(void);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlMCR &);

    virtual void 
    Free(void);
    
    XString 
    AssembleChoice(XmlContext& ctx);

    wctp_parse_t 
    ParseChoice(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctpXmlMessageText    *m_pMessageText;
    TStack<wctpXmlChoice>  m_vChoice;
};
#endif
