/* File:  wctpXmlMessageReceived.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_MESSAGE_RECEIVED_HPP
#define WCTP_XML_MESSAGE_RECEIVED_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlFailure.hpp"
#include "wctpXmlSuccess.hpp"

class wctpXmlMessageReceived : wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlMessageReceived(); 
    wctpXmlMessageReceived(const wctpXmlMessageReceived &); 
    ~wctpXmlMessageReceived(); 
    wctpXmlMessageReceived& operator=(const wctpXmlMessageReceived &);
    
    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // MessageReceived Type
    wctp_xml_t 
    GetMessageReceivedType(void);

    ////////////////////////////
    // Failure Child Element (holds only one of set)
    wctpXmlFailure* 
    GetFailure(void);
    void // specify to hold this item from the set
    HoldFailure(void);

    ////////////////////////////
    // Success Child Element (holds only one of set)
    wctpXmlSuccess* 
    GetSuccess(void);
    void // specify to hold this item from the set
    HoldSuccess(void);

    ////////////////////////////
    // sequenceNo attribute
    const char * 
    GetSequenceNo(void) const;
    void  
    SetSequenceNo(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlMessageReceived &);

    virtual void 
    Free(void);
    
    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctp_xml_t      m_MessageReceivedType;
    wctpXmlFailure *m_pFailure;
    wctpXmlSuccess *m_pSuccess;
    XString         m_sequenceNo;
    
};
#endif
