/* File:  wctpXmlPayload.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_PAYLOAD_HPP
#define WCTP_XML_PAYLOAD_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlMCR.hpp"
#include "wctpXmlAlphanumeric.hpp"
#include "wctpXmlTransparentData.hpp"


class wctpXmlPayload : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlPayload(); 
    wctpXmlPayload(const wctpXmlPayload &); 
    ~wctpXmlPayload(); 
    wctpXmlPayload& operator=(const wctpXmlPayload &);

    ////////////////////////////
    // reset object to default state 
    void 
    Clear(void);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);

    ////////////////////////////
    // Payload Type
    wctp_xml_t 
    GetPayloadType(void);

    ////////////////////////////
    // MCR Child Element (holds only one of set)
    wctpXmlMCR* 
    GetMCR(void);
    void // specify to hold this item from the set
    HoldMCR(void);

    ////////////////////////////
    // Alphanumeric Child Element (holds only one of set)
    wctpXmlAlphanumeric* 
    GetAlphanumeric(void);
    void // specify to hold this item from the set
    HoldAlphanumeric(void);

    ////////////////////////////
    // TransparentData Child Element (holds only one of set)
    wctpXmlTransparentData* 
    GetTransparentData(void);
    void // specify to hold this item from the set
    HoldTransparentData(void);
	
   	
protected:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlPayload &);

    virtual void 
    Free(void);

    virtual wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctp_xml_t              m_PayloadType;
    wctpXmlMCR             *m_pMCR;
    wctpXmlAlphanumeric    *m_pAlphanumeric;
    wctpXmlTransparentData *m_pTransparentData;

};
#endif
