/* File:  wctpXmlPollForMessages.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_POLL_FOR_MESSAGES_HPP
#define WCTP_XML_POLL_FOR_MESSAGES_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlMessageReceived.hpp"

class wctpXmlPollForMessages : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlPollForMessages(); 
    wctpXmlPollForMessages(const wctpXmlPollForMessages &); 
    ~wctpXmlPollForMessages(); 
    wctpXmlPollForMessages& operator=(const wctpXmlPollForMessages &);

    ////////////////////////////
    // reset object to default state 
    virtual void 
    Clear(void);

    ////////////////////////////
    // MessageReceived Child Elements (zero or more)

    // erase all MessageReceived elements
    void 
    ClearMessageReceived(void);  
    // get MessageReceived element at given position
    // ---- pos is a zero-based index 
    wctpXmlMessageReceived* 
    GetMessageReceived(int pos); 
    // add MessageReceived element at end of array
    wctpXmlMessageReceived* 
    AddMessageReceived(void);
    // erase MessageReceived element at given position
    // ---- pos is a zero-based index 
    void  
    EraseMessageReceived(int pos);
    // return number of MessageReceived elements
    int 
    NumMessageReceived(void);

    ////////////////////////////
    // maxMessagesInBatch attribute
    const char * 
    GetMaxMessagesInBatch(void) const;
    void  
    SetMaxMessagesInBatch(const char *);

    ////////////////////////////
    // pollerID attribute
    const char * 
    GetPollerID(void) const;
    void  
    SetPollerID(const char *);

    ////////////////////////////
    // securityCode attribute
    const char * 
    GetSecurityCode(void) const;
    void  
    SetSecurityCode(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlPollForMessages &);

    virtual void 
    Free(void);
	
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    XString 
    AssembleMessageReceived(XmlContext& ctx);

    wctp_parse_t 
    ParseMessageReceived(XString& msg, XmlNode *pTree, int& pos);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    // attributes
    TStack<wctpXmlMessageReceived> m_vMessageReceived;
    XString                        m_maxMessagesInBatch;
    XString                        m_pollerID;
    XString                        m_securityCode;
    
};
#endif
