/* File:  wctpXmlPollResponse.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_POLL_RESPONSE_HPP
#define WCTP_XML_POLL_RESPONSE_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlMessage.hpp"
#include "wctpXmlNoMessages.hpp"

class wctpXmlPollResponse : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlPollResponse(); 
    wctpXmlPollResponse(const wctpXmlPollResponse &); 
    ~wctpXmlPollResponse(); 
    wctpXmlPollResponse& operator=(const wctpXmlPollResponse &);

    ////////////////////////////
    // reset object to default state 
    virtual void 
    Clear(void);

    ////////////////////////////
    // PollResponse Type
    wctp_xml_t 
    GetPollResponseType(void);

    ////////////////////////////
    // Message Child Elements (one or more)

    // erase all Message elements
    void 
    ClearMessage(void);  
    // get Message element at given position
    // ---- pos is a zero-based index 
    wctpXmlMessage* 
    GetMessage2(int pos); 
    // add Message element at end of array
    wctpXmlMessage* 
    AddMessage(void);
    // erase Message element at given position
    // ---- pos is a zero-based index 
    void  
    EraseMessage(int pos);
    // return number of Message elements
    int 
    NumMessage(void);
    void // specify to hold this item from the set
    HoldMessage(void);

    ////////////////////////////
    // NoMessages Child Element (holds only one of set)
    wctpXmlNoMessages* 
    GetNoMessages(void);
    void // specify to hold this item from the set
    HoldNoMessages(void);

    ////////////////////////////
    // minNextPollInterval attribute
    const char * 
    GetMinNextPollInterval(void) const;
    void  
    SetMinNextPollInterval(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlPollResponse &);

    virtual void 
    Free(void);

    virtual XString 
    AssembleOperation(XmlContext& ctx);

    virtual wctp_parse_t 
    ParseObject (XString& msg, XmlNode *pTree);
    
    XString 
    AssembleMessage(XmlContext& ctx);

    wctp_parse_t 
    ParseMessage(XString& msg, XmlNode *pTree, int& pos);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctp_xml_t             m_PollResponseType;
    TStack<wctpXmlMessage> m_vMessage;
    wctpXmlNoMessages     *m_pNoMessages;
    XString                m_minNextPollInterval;
  
};
#endif
