/* File:  wctpXmlVersionQueryResponse.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_VERSION_QUERY_RESPONSE_HPP
#define WCTP_XML_VERSION_QUERY_RESPONSE_HPP

#include "wctpXmlVersionContactInfo.hpp"
#include "wctpXmlVersionDTDSupport.hpp"
#include "wctpXmlFailure.hpp"
#include "wctpXmlOperation.hpp"

class wctpXmlVersionQueryResponse : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlVersionQueryResponse(); 
    wctpXmlVersionQueryResponse(const wctpXmlVersionQueryResponse &); 
    ~wctpXmlVersionQueryResponse(); 
    wctpXmlVersionQueryResponse& operator=(const wctpXmlVersionQueryResponse &);

    ////////////////////////////
    // reset object to default state 
    virtual void 
    Clear(void);

    ////////////////////////////
    // VersionQueryResponse Type
    wctp_xml_t 
    GetVersionQueryResponseType(void);

    ////////////////////////////
    // VersionContactInfo Child Elements (holds only one of set)

    // get VersionContactInfo element at given position
    wctpXmlVersionContactInfo* 
    GetVersionContactInfo(void); 
    void // specify to hold this item from the set
    HoldVersionContactInfo(void);

    ////////////////////////////
    // VersionDTDSupport Child Element (one or more) 
	
    // erase all VersionDTDSupport elements
    void 
    ClearVersionDTDSupport(void); 
    // add VersionDTDSupport element at end of array
    wctpXmlVersionDTDSupport* 
    AddVersionDTDSupport(void);
    // erase VersionDTDSupport element at given position
    // ---- pos is a zero-based index 
    void  
    EraseVersionDTDSupport(int pos);
    // return number of VersionDTDSupport elements
    int 
    NumVersionDTDSupport(void);
    // get VersionDTDSupport element at given position
    // ---- pos is a zero-based index 
    wctpXmlVersionDTDSupport* 
    GetVersionDTDSupport(int pos); 	
    void // specify to hold this item from the set
    HoldVersionDTDSupport(void);

    ////////////////////////////
    // Failure Child Element (holds only one of set)
    wctpXmlFailure* 
    GetFailure(void);
    void // specify to hold this item from the set
    HoldFailure(void);

    ////////////////////////////
    // responder attribute
    const char * 
    GetResponder(void) const;
    void  
    SetResponder(const char *);

    ////////////////////////////
    // inquirer attribute
    const char * 
    GetInquirer(void) const;
    void  
    SetInquirer(const char *);

    ////////////////////////////
    // dateTimeOfRsp attribute
    const char * 
    GetDateTimeOfRsp(void) const;
    void  
    SetDateTimeOfRsp(const char *);

    ////////////////////////////
    // dateTimeOfReq attribute
    const char * 
    GetDateTimeOfReq(void) const;
    void  
    SetDateTimeOfReq(const char *);

    ////////////////////////////
    // invalidAfter attribute
    const char * 
    GetInvalidAfter(void) const;
    void  
    SetInvalidAfter(const char *);

    ////////////////////////////
    // listDTDs attribute
    const char * 
    GetListDTD(void) const;
    void  
    SetListDTD(const char *);

    ////////////////////////////
    // listConfiguration attribute
    const char * 
    GetListConfig(void) const;
    void  
    SetListConfig(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlVersionQueryResponse &);

    virtual void 
    Free(void);

    virtual XString 
    AssembleOperation(XmlContext& ctx);

    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    XString 
    AssembleVersionDTDSupport(XmlContext& ctx);

    wctp_parse_t 
    ParseVersionDTDSupport(XString& msg, XmlNode *pTree, int& pos);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);
    
    // attributes
    wctp_xml_t                       m_VersionQueryResponseType;
    TStack<wctpXmlVersionDTDSupport> m_vDTDSupport;
    wctpXmlVersionContactInfo       *m_pContactInfo;
    wctpXmlFailure                  *m_pFailure;
    XString                          m_dateTimeReq;
    XString                          m_dateTimeRsp;
    XString                          m_inquirer;
    XString                          m_responder;
    XString                          m_invalidAfter;
    XString                          m_listConfig;
    XString                          m_listDTD;
    XString                          m_email;
    XString                          m_phone;
    XString                          m_www;
    XString                          m_info;
    XString                          m_wctpVersion;
    XString                          m_wctpToken;
};
#endif
