/* File:  XmlContext.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "XmlContext.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//////////////////////////////////////////////////////////////////////////////
//

XmlContext::XmlContext()
{
    m_indent_level = 0;
}

//////////////////////////////////////////////////////////////////////////////
//

XmlContext::~XmlContext()
{
}

//////////////////////////////////////////////////////////////////////////////
//

XString 
XmlContext::XmlEncode(const XString& src)
{
    XString dest;
    int num = src.size();
    int i;
    for(i=0; i<num; i++)
    {
        if(src[i] == '&')
        {
            dest += "&amp;";
        }
        else if(src[i] == '<')
        {
            dest += "&lt;";
        }
        else if(src[i] == '>')
        {
            dest += "&gt;";
        }
        else if(src[i] == '\'')
        {
            dest += "&apos;";
        }
        else if(src[i] == '\"')
        {
            dest += "&quot;";
        }
        else if(isprint(src[i]))
        {
            dest += src[i];
        }
        else if (src[i] == 9  ||    // horizontal tab
                 src[i] == 10 ||    // line feed
                 src[i] == 13)      // carriage return 
        {
            // these are the only valid non-printable characters
            char buf[16];
            sprintf(buf,"&#x%X;", src[i]);
            dest += buf;
        }
    }    // else - throw them away 

    return dest;
}

//////////////////////////////////////////////////////////////////////////////
//

XString 
XmlContext::XmlAttribute(
    const char *key, 
    const char *val)
{
    XString str(val,strlen(val));
    return XmlAttribute(key, str);
}

//////////////////////////////////////////////////////////////////////////////
//

XString 
XmlContext::XmlAttribute(
    const char *key, 
    const XString& val)
{
    XString dest;
    if(val.size() > 0)
    {
        dest += Indent();
        dest += key;
        dest += "=\"";
        dest += XmlEncode(val);
        dest += "\"";
        dest += "\n";
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
//

XString 
XmlContext::XmlPcdata(const XString &pcdata)
{
    XString text,dest;

    // remove leading and trailing spaces (if any)
    // including ending newline (if any)

    text = pcdata.both_trim();

    // don't indent if text has multiple lines or is long

    if(!text.is_multiline() && 0 < text.length() && text.length() <= 50)
    {
        dest += Indent();
    }

    // don't add a blank line if no pcdata

    if(text.length() > 0)
    {
        dest += text;
    
    }

    dest = XmlEncode(dest);

    dest += "\n";
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
//

XString
XmlContext::OpenIndent(const char *txt)
{
    XString dest;
    dest += IndentText(txt);
    m_indent_level += 4;
    return dest; 
}

//////////////////////////////////////////////////////////////////////////////
//

XString
XmlContext::Indent(void)
{
    XString dest;
    int i;
    for(i=0; i < m_indent_level; i++)
    {
        dest += " ";
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
//

XString
XmlContext::IndentText(const char *txt)
{
    XString dest;
    dest += Indent();
    dest += txt;
    dest += "\n";
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
//

XString
XmlContext::CloseIndent(const char *txt)
{
    m_indent_level -= 4;
    return IndentText(txt);
}

