/* File:  wctpXmlClientMessageControl.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlClientMessageControl.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientMessageControl::wctpXmlClientMessageControl()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientMessageControl::wctpXmlClientMessageControl(
    const wctpXmlClientMessageControl &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientMessageControl::~wctpXmlClientMessageControl()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientMessageControl&
wctpXmlClientMessageControl::operator=(
    const wctpXmlClientMessageControl &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientMessageControl::Init(void)
{
    wctpXmlOperation::Init();

    m_allowResponse = WCTP_BOOL_TRUE;
    m_allowTruncation = WCTP_BOOL_DEFAULT;
    m_deliveryAfter = "";
    m_deliveryBefore = "";
    m_deliveryPriority = WCTP_PRIORITY_DEFAULT;
    m_notifyWhenDelivered = WCTP_BOOL_DEFAULT;
    m_notifyWhenQueued = WCTP_BOOL_DEFAULT;
    m_notifyWhenRead = WCTP_BOOL_DEFAULT;
    m_preformatted = WCTP_BOOL_DEFAULT;
    m_sendResponsesToID = "";

    m_title = CLIENT_MESSAGE_CONTROL_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientMessageControl::Copy(const wctpXmlClientMessageControl &rval)
{
    //
    // Copy member variables
    //
    wctpXmlOperation::Copy(rval);

    m_allowResponse = rval.m_allowResponse;
    m_allowTruncation = rval.m_allowTruncation;
    m_deliveryAfter = rval.m_deliveryAfter;
    m_deliveryBefore = rval.m_deliveryBefore;
    m_deliveryPriority = rval.m_deliveryPriority;
    m_notifyWhenDelivered = rval.m_notifyWhenDelivered;
    m_notifyWhenQueued = rval.m_notifyWhenQueued;
    m_notifyWhenRead = rval.m_notifyWhenRead;
    m_preformatted = rval.m_preformatted;
    m_sendResponsesToID = rval.m_sendResponsesToID;
}



//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientMessageControl::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientMessageControl");
    dest += ctx.XmlAttribute(
        "allowResponse", wctpXml::bool_to_str(m_allowResponse));
    dest += ctx.XmlAttribute(
        "allowTruncation", wctpXml::bool_to_str(m_allowTruncation));
    dest += ctx.XmlAttribute(
        "deliveryAfter",m_deliveryAfter);
    dest += ctx.XmlAttribute(
        "deliveryBefore",m_deliveryBefore);
    dest += ctx.XmlAttribute(
        "deliveryPriority", wctpXml::priority_to_str(m_deliveryPriority));
    dest += ctx.XmlAttribute(
        "notifyWhenDelivered", wctpXml::bool_to_str(m_notifyWhenDelivered));
    dest += ctx.XmlAttribute(
        "notifyWhenQueued", wctpXml::bool_to_str(m_notifyWhenQueued));
    dest += ctx.XmlAttribute(
        "notifyWhenRead", wctpXml::bool_to_str(m_notifyWhenRead));
    dest += ctx.XmlAttribute(
        "preformatted", wctpXml::bool_to_str(m_preformatted));
    dest += ctx.XmlAttribute(
        "sendResponsesToID",m_sendResponsesToID);
    dest += ctx.CloseIndent("/>");
    return dest;     
}



//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlClientMessageControl::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "allowResponse")
    {
        m_allowResponse = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "allowTruncation")
    {
        m_allowTruncation = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "deliveryAfter")
    {
        m_deliveryAfter = val;
    }
    else if(key == "deliveryBefore")
    {
        m_deliveryBefore = val;
    }
    else if(key == "deliveryPriority")
    {
        m_deliveryPriority = wctpXml::str_to_priority(val.c_str());
    }
    else if(key == "notifyWhenDelivered")
    {
        m_notifyWhenDelivered = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "notifyWhenQueued")
    {
        m_notifyWhenQueued = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "notifyWhenRead")
    {
        m_notifyWhenRead = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "preformatted")
    {
        m_preformatted = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "sendResponsesToID")
    {
        m_sendResponsesToID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-ClientMessageControl";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientMessageControl::ParseObject(XString& msg, XmlNode *pTree)
{
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientMessageControl")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientMessageControl, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientMessageControl does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-ClientMessageControl element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_allowResponse attribute

wctp_bool_t 
wctpXmlClientMessageControl::GetAllowResponse(void) const 
{
    return m_allowResponse;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_allowResponse attribute

void  
wctpXmlClientMessageControl::SetAllowResponse(wctp_bool_t newValue)
{
    m_allowResponse = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_allowTruncation attribute

wctp_bool_t 
wctpXmlClientMessageControl::GetAllowTruncation(void) const 
{
    return m_allowTruncation;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_allowTruncation attribute

void  
wctpXmlClientMessageControl::SetAllowTruncation(wctp_bool_t newValue)
{
    m_allowTruncation = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_deliveryAfter attribute

const char * 
wctpXmlClientMessageControl::GetDeliveryAfter(void) const 
{
    return m_deliveryAfter;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_deliveryAfter attribute

void  
wctpXmlClientMessageControl::SetDeliveryAfter(const char * newValue)
{
    m_deliveryAfter = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_deliveryBefore attribute

const char * 
wctpXmlClientMessageControl::GetDeliveryBefore(void) const 
{
    return m_deliveryBefore;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_deliveryBefore attribute

void  
wctpXmlClientMessageControl::SetDeliveryBefore(const char * newValue)
{
    m_deliveryBefore = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_deliveryPriority attribute

wctp_priority_t 
wctpXmlClientMessageControl::GetDeliveryPriority(void) const 
{
    return m_deliveryPriority;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_deliveryPriority attribute

void  
wctpXmlClientMessageControl::SetDeliveryPriority(wctp_priority_t newValue)
{
    m_deliveryPriority = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_notifyWhenDelivered attribute

wctp_bool_t 
wctpXmlClientMessageControl::GetNotifyWhenDelivered(void) const 
{
    return m_notifyWhenDelivered;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_notifyWhenDelivered attribute

void  
wctpXmlClientMessageControl::SetNotifyWhenDelivered(wctp_bool_t newValue)
{
    m_notifyWhenDelivered = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_notifyWhenQueued attribute

wctp_bool_t 
wctpXmlClientMessageControl::GetNotifyWhenQueued(void) const 
{
    return m_notifyWhenQueued;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_notifyWhenQueued attribute

void  
wctpXmlClientMessageControl::SetNotifyWhenQueued(wctp_bool_t newValue)
{
    m_notifyWhenQueued = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_notifyWhenRead attribute

wctp_bool_t 
wctpXmlClientMessageControl::GetNotifyWhenRead(void) const 
{
    return m_notifyWhenRead;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_notifyWhenRead attribute

void  
wctpXmlClientMessageControl::SetNotifyWhenRead(wctp_bool_t newValue)
{
    m_notifyWhenRead = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_preformatted attribute

wctp_bool_t 
wctpXmlClientMessageControl::GetPreformatted(void) const 
{
    return m_preformatted;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_preformatted attribute

void  
wctpXmlClientMessageControl::SetPreformatted(wctp_bool_t newValue)
{
    m_preformatted = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_sendResponsesToID attribute

const char * 
wctpXmlClientMessageControl::GetSendResponsesToID(void) const 
{
    return m_sendResponsesToID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_sendResponsesToID attribute

void  
wctpXmlClientMessageControl::SetSendResponsesToID(const char * newValue)
{
    m_sendResponsesToID = newValue;
}

