/* File:  wctpXmlClientQuery.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <stdio.h>
#include "wctpXmlClientQuery.hpp"

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientQuery::wctpXmlClientQuery()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientQuery::wctpXmlClientQuery(
    const wctpXmlClientQuery &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientQuery::~wctpXmlClientQuery()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientQuery&
wctpXmlClientQuery::operator=(
    const wctpXmlClientQuery &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientQuery::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_CLIENT_QUERY;

    m_recipientID = "";
    m_senderID = "";
    m_trackingNumber = "";

    m_title = CLIENT_QUERY_STRING;
    
    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;

}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientQuery::Copy(const wctpXmlClientQuery &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy member variables
    //

    m_recipientID = rval.m_recipientID;
    m_senderID = rval.m_senderID;
    m_trackingNumber = rval.m_trackingNumber;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlClientQuery::Free(void)
{
    wctpXmlOperation::Free();
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlClientQuery::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientQuery::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientQuery");
    dest += ctx.XmlAttribute(
        "recipientID",m_recipientID);
    dest += ctx.XmlAttribute(
        "senderID",m_senderID);
    dest += ctx.XmlAttribute(
        "trackingNumber",m_trackingNumber);
    dest += ctx.CloseIndent("/>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlClientQuery::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "recipientID")
    {
        m_recipientID = val;
    }
    else if(key == "senderID")
    {
        m_senderID = val;
    }
    else if(key == "trackingNumber")
    {
        m_trackingNumber = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-ClientQuery";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientQuery::ParseObject(XString& msg,XmlNode *pTree)
{
    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientQuery")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientQuery, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientQuery does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-ClientQuery element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_recipientID attribute

const char * 
wctpXmlClientQuery::GetRecipientID(void) const 
{
    return m_recipientID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_recipientID attribute

void  
wctpXmlClientQuery::SetRecipientID(const char * newValue)
{
    m_recipientID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_senderID attribute

const char * 
wctpXmlClientQuery::GetSenderID(void) const 
{
    return m_senderID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_senderID attribute

void  
wctpXmlClientQuery::SetSenderID(const char * newValue)
{
    m_senderID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_trackingNumber attribute

const char * 
wctpXmlClientQuery::GetTrackingNumber(void) const 
{
    return m_trackingNumber;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_trackingNumber attribute

void  
wctpXmlClientQuery::SetTrackingNumber(const char * newValue)
{
    m_trackingNumber = newValue;
}


