/* File:  wctpXmlClientResponseHeader.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlClientResponseHeader.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientResponseHeader::wctpXmlClientResponseHeader()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientResponseHeader::wctpXmlClientResponseHeader(
    const wctpXmlClientResponseHeader &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientResponseHeader::~wctpXmlClientResponseHeader()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientResponseHeader&
wctpXmlClientResponseHeader::operator=(
    const wctpXmlClientResponseHeader &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientResponseHeader::Init(void)
{
    wctpXmlOperation::Init();

    m_pOriginator = new wctpXmlOriginator;
    m_pRecipient = new wctpXmlRecipient;
    m_respondingToTimestamp = "";
    m_responseTimestamp = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientResponseHeader::Copy(const wctpXmlClientResponseHeader &rval)
{
    //
    // Copy pOriginator object if available (should always hold one)
    //
    
    wctpXmlOperation::Copy(rval);

    if(rval.m_pOriginator != NULL)
    {
        m_pOriginator = new wctpXmlOriginator(
            *rval.m_pOriginator);
    }
    else
    {
        m_pOriginator = new wctpXmlOriginator;
    }

    //
    // Copy pRecipient object if available (should always hold one)
    //

    if(rval.m_pRecipient != NULL)
    {
        m_pRecipient = new wctpXmlRecipient(
            *rval.m_pRecipient);
    }
    else
    {
        m_pRecipient = new wctpXmlRecipient;
    }

    //
    // Copy member variables
    //

    m_respondingToTimestamp = rval.m_respondingToTimestamp;
    m_responseTimestamp = rval.m_responseTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlClientResponseHeader::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pOriginator != NULL) 
    {
        delete m_pOriginator;
        m_pOriginator = NULL;
    }
    if(m_pRecipient != NULL) 
    {
        delete m_pRecipient;
        m_pRecipient = NULL;
    }
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientResponseHeader::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientResponseHeader");
    dest += ctx.XmlAttribute(
        "respondingToTimestamp",m_respondingToTimestamp);
    dest += ctx.XmlAttribute(
        "responseTimestamp",m_responseTimestamp);
    dest += ctx.IndentText(">");
    if(m_pOriginator != NULL)
    {
        dest += m_pOriginator->AssembleOperation(ctx);
    }
    if(m_pRecipient != NULL)
    {
        dest += m_pRecipient->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-ClientResponseHeader>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlClientResponseHeader::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "respondingToTimestamp")
    {
        m_respondingToTimestamp = val;
    }
    else if(key == "responseTimestamp")
    {
        m_responseTimestamp = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-ClientResponseHeader";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientResponseHeader::ParseObject(XString& msg, XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientResponseHeader")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientResponseHeader, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientResponseHeader does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-Originator object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pOriginator->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Originator element expected in ";
        msg += "wctp-ClientResponseHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Recipient object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pRecipient->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Recipient element expected in ";
        msg += "wctp-ClientResponseHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-ClientResponseHeader element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Originator object
// This function will never return NULL unless new fails...

wctpXmlOriginator*
wctpXmlClientResponseHeader::GetOriginator(void) 
{
    if(m_pOriginator == NULL)
    {
        m_pOriginator = new wctpXmlOriginator;
    }
    return m_pOriginator;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Recipient object
// This function will never return NULL unless new fails...

wctpXmlRecipient*
wctpXmlClientResponseHeader::GetRecipient(void) 
{
    if(m_pRecipient == NULL)
    {
        m_pRecipient = new wctpXmlRecipient;
    }
    return m_pRecipient;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_respondingToTimestamp attribute

const char * 
wctpXmlClientResponseHeader::GetRespondingToTimestamp(void) const 
{
    return m_respondingToTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_respondingToTimestamp attribute

void  
wctpXmlClientResponseHeader::SetRespondingToTimestamp(const char * newValue)
{
    m_respondingToTimestamp = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_responseTimestamp attribute

const char * 
wctpXmlClientResponseHeader::GetResponseTimestamp(void) const 
{
    return m_responseTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_responseTimestamp attribute

void  
wctpXmlClientResponseHeader::SetResponseTimestamp(const char * newValue)
{
    m_responseTimestamp = newValue;
}

