/* File:  wctpXmlClientStatusInfo.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlClientStatusInfo.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientStatusInfo::wctpXmlClientStatusInfo()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientStatusInfo::wctpXmlClientStatusInfo(
    const wctpXmlClientStatusInfo &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientStatusInfo::~wctpXmlClientStatusInfo()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientStatusInfo&
wctpXmlClientStatusInfo::operator=(
    const wctpXmlClientStatusInfo &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientStatusInfo::Init(void)
{
    wctpXmlOperation::Init();

    m_ClientStatusInfoType = WCTP_XML_UNDEFINED;
    m_pClientResponseHeader = new wctpXmlClientResponseHeader;
    m_pFailure = NULL;
    m_pNotification = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientStatusInfo::Copy(const wctpXmlClientStatusInfo &rval)
{
    //
    // Copy object type
    //

    wctpXmlOperation::Copy(rval);

    m_ClientStatusInfoType = rval.m_ClientStatusInfoType;

    //
    // Copy pClientResponseHeader object if available (should always hold one)
    //

    if(rval.m_pClientResponseHeader != NULL)
    {
        m_pClientResponseHeader = new wctpXmlClientResponseHeader(
            *rval.m_pClientResponseHeader);
    }
    else
    {
        m_pClientResponseHeader = new wctpXmlClientResponseHeader;
    }

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy pNotification object if available (maybe holds)
    //

    if(rval.m_pNotification != NULL)
    {
        m_pNotification = new wctpXmlNotification(
            *rval.m_pNotification);
    }
    else
    {
        m_pNotification = NULL;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlClientStatusInfo::Free(void)
{

    wctpXmlOperation::Free();

    if(m_pClientResponseHeader != NULL) 
    {
        delete m_pClientResponseHeader;
        m_pClientResponseHeader = NULL;
    }
    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
    if(m_pNotification != NULL) 
    {
        delete m_pNotification;
        m_pNotification = NULL;
    }
}



//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientStatusInfo::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientStatusInfo>");
    if(m_pClientResponseHeader != NULL)
    {
        dest += m_pClientResponseHeader->AssembleOperation(ctx);
    }
    if(m_ClientStatusInfoType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    if(m_ClientStatusInfoType == WCTP_XML_NOTIFICATION && 
       m_pNotification != NULL)
    {
        dest += m_pNotification->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-ClientStatusInfo>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlClientStatusInfo::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-ClientStatusInfo";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-Failure")
    {
        HoldFailure();
        pos++;
        res = m_pFailure->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-Notification")
    {
        HoldNotification();
        pos++;
        res = m_pNotification->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-ClientStatusInfo";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientStatusInfo::ParseObject(XString& msg,XmlNode *pTree)
{
  
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientStatusInfo")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientStatusInfo, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-ClientStatusInfo does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientStatusInfo does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-ClientResponseHeader object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pClientResponseHeader->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-ClientResponseHeader element expected in ";
        msg += "wctp-ClientStatusInfo element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-ClientStatusInfo element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlClientStatusInfo::GetClientStatusInfoType(void)
{
    return m_ClientStatusInfoType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientResponseHeader object
// This function will never return NULL unless new fails...

wctpXmlClientResponseHeader*
wctpXmlClientStatusInfo::GetClientResponseHeader(void) 
{
    if(m_pClientResponseHeader == NULL)
    {
        m_pClientResponseHeader = new wctpXmlClientResponseHeader;
    }
    return m_pClientResponseHeader;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlClientStatusInfo::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlClientStatusInfo::HoldFailure(void) 
{
    m_ClientStatusInfoType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Notification object
// This function will never return NULL unless new fails...

wctpXmlNotification*
wctpXmlClientStatusInfo::GetNotification(void) 
{
    if(m_pNotification == NULL)
    {
        m_pNotification = new wctpXmlNotification;
    }
    return m_pNotification;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Notification object 

void
wctpXmlClientStatusInfo::HoldNotification(void) 
{
    m_ClientStatusInfoType = WCTP_XML_NOTIFICATION;
    if(m_pNotification == NULL)
    {
        m_pNotification = new wctpXmlNotification;
    }
}

