/* File:  wctpXmlConfirmation.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlConfirmation.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlConfirmation::wctpXmlConfirmation()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlConfirmation::wctpXmlConfirmation(
    const wctpXmlConfirmation &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlConfirmation::~wctpXmlConfirmation()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlConfirmation&
wctpXmlConfirmation::operator=(
    const wctpXmlConfirmation &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlConfirmation::Init(void)
{
    wctpXmlOperation::Init();
    

    m_ConfirmationType = WCTP_XML_UNDEFINED;
    m_pFailure = NULL;
    m_pSuccess = NULL;
        
    m_title = CONFIRMATION_STRING;
    
    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;

}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlConfirmation::Copy(const wctpXmlConfirmation &rval)
{
    wctpXmlOperation::Copy(rval);
    //
    // Copy object type
    //

    m_ConfirmationType = rval.m_ConfirmationType;

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy pSuccess object if available (maybe holds)
    //

    if(rval.m_pSuccess != NULL)
    {
        m_pSuccess = new wctpXmlSuccess(
            *rval.m_pSuccess);
    }
    else
    {
        m_pSuccess = NULL;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlConfirmation::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
    if(m_pSuccess != NULL) 
    {
        delete m_pSuccess;
        m_pSuccess = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlConfirmation::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlConfirmation::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-Confirmation>");
    if(m_ConfirmationType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    if(m_ConfirmationType == WCTP_XML_SUCCESS && 
       m_pSuccess != NULL)
    {
        dest += m_pSuccess->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-Confirmation>");
    return dest;     
}



//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlConfirmation::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-Confirmation";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-Failure")
    {
        HoldFailure();
        pos++;
        res = m_pFailure->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-Success")
    {
        HoldSuccess();
        pos++;
        res = m_pSuccess->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-Confirmation";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlConfirmation::ParseObject(XString& msg, XmlNode *pTree)
{
    int pos = 0;

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-Confirmation")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-Confirmation, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-Confirmation does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-Confirmation does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-Confirmation element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlConfirmation::GetConfirmationType(void)
{
    return m_ConfirmationType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlConfirmation::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlConfirmation::HoldFailure(void) 
{
    m_ConfirmationType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Success object
// This function will never return NULL unless new fails...

wctpXmlSuccess*
wctpXmlConfirmation::GetSuccess(void) 
{
    if(m_pSuccess == NULL)
    {
        m_pSuccess = new wctpXmlSuccess;
    }
    return m_pSuccess;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Success object 

void
wctpXmlConfirmation::HoldSuccess(void) 
{
    m_ConfirmationType = WCTP_XML_SUCCESS;
    if(m_pSuccess == NULL)
    {
        m_pSuccess = new wctpXmlSuccess;
    }
}

