/* File:  wctpXmlMCR.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlMCR.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlMCR::wctpXmlMCR()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlMCR::wctpXmlMCR(
    const wctpXmlMCR &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlMCR::~wctpXmlMCR()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlMCR&
wctpXmlMCR::operator=(const wctpXmlMCR &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlMCR::Init(void)
{
    wctpXmlOperation::Init();

    m_pMessageText = new wctpXmlMessageText;

    m_xmlPrefix= "wctp";
    m_title = MCR_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}
    

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlMCR::Copy(const wctpXmlMCR &rval)
{
    //
    // Copy pMessageText object if available (should always hold one)
    //

    wctpXmlOperation::Copy(rval);

    if(rval.m_pMessageText != NULL)
    {
        m_pMessageText = new wctpXmlMessageText(
            *rval.m_pMessageText);
    }
    else
    {
        m_pMessageText = new wctpXmlMessageText;
    }

    //
    // Copy pChoice object(s) if available (holds one or more)
    //

    if(rval.m_vChoice.size() > 0)
    {
        int i;
        for(i=0; i<rval.m_vChoice.size(); i++)
        {
            wctpXmlChoice *pChoice = 
                new wctpXmlChoice(
                *(rval.m_vChoice[i]));
            m_vChoice.push(pChoice);
        }
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlMCR::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pMessageText != NULL) 
    {
        delete m_pMessageText;
        m_pMessageText = NULL;
    }
    m_vChoice.destroy();
}


//////////////////////////////////////////////////////////////////////////////
// Construct an XML string for Choice object(s)

XString
wctpXmlMCR::AssembleChoice(XmlContext& ctx)
{
    XString dest;
    int i;
    for(i=0; i<m_vChoice.size(); i++) 
    {
        dest += m_vChoice[i]->AssembleOperation(ctx); 
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlMCR::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    XString temp;

    temp = "<";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.OpenIndent(temp);
    if(m_pMessageText != NULL)
    {
        dest += m_pMessageText->AssembleOperation(ctx);
    }
    dest += AssembleChoice(ctx);

    temp = "</";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.CloseIndent(temp);

    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse one or more Choice object(s)

wctp_parse_t
wctpXmlMCR::ParseChoice( XString& msg, XmlNode *pTree,
    int& pos)
{
    bool flag = false;
    while(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if (pChild->element == "wctp-Choice")
        {
            flag = true;
            pos++;
            wctpXmlChoice *pChoice = AddChoice();
            wctp_parse_t res = pChoice->ParseObject(msg, pChild); 
            if(res != WCTP_PARSE_OKAY)
            {
                return res; 
            }
        }
    }
    if(!flag) 
    {
        msg = "expect at least one wctp-Choice inside ";
        msg += m_xmlOperation;
        return WCTP_PARSE_ERROR;        
    }
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlMCR::ParseObject(XString& msg,XmlNode *pTree)
{
 
    //
    // make sure the element matches me
    //

    if(m_xmlOperation != pTree->element)
    {
        msg = "element mismatch, expected ";
        msg += m_xmlOperation;
        msg += ", but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = m_xmlOperation;
        msg += " does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = m_xmlOperation;
        msg += " does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-MessageText object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pMessageText->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-MessageText element expected in ";
        msg += m_xmlOperation;
        msg += " element";
        return WCTP_PARSE_ERROR;
    }

     //
     // Load in one or more Choice objects
     //

     res = ParseChoice(msg,pTree,pos);
     if(res != WCTP_PARSE_OKAY)
     {
         return res;
     }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = m_xmlOperation;
        msg += " element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to MessageText object
// This function will never return NULL unless new fails...

wctpXmlMessageText*
wctpXmlMCR::GetMessageText(void) 
{
    if(m_pMessageText == NULL)
    {
        m_pMessageText = new wctpXmlMessageText;
        m_pMessageText->SetPrefix(m_xmlPrefix);
    }
    return m_pMessageText;
}

//////////////////////////////////////////////////////////////////////////////
// Clear all Choice elements

void
wctpXmlMCR::ClearChoice(void) 
{
    m_vChoice.destroy();
}

//////////////////////////////////////////////////////////////////////////////
// Get Choice element at given position

wctpXmlChoice *
wctpXmlMCR::GetChoice(int pos)
{
    if(pos >= 0 && pos < m_vChoice.size())
    {
        return m_vChoice[pos];
    }
    return NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Add Choice element to end of array

wctpXmlChoice *
wctpXmlMCR::AddChoice(void)
{
    wctpXmlChoice *pChoice =  new wctpXmlChoice;
    pChoice->SetPrefix(m_xmlPrefix);


    m_vChoice.push(pChoice);
    return pChoice;
}

//////////////////////////////////////////////////////////////////////////////
// Erase Choice element at given position

void
wctpXmlMCR::EraseChoice(int pos)
{
    wctpXmlChoice *pChoice = 
        m_vChoice.remove(pos);
    delete pChoice;
    pChoice = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Return number of Choice elements

int
wctpXmlMCR::NumChoice(void)
{
    return m_vChoice.size();
}

