/* File:  wctpXmlMessage.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlMessage.hpp"
#include <stdio.h>


//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlMessage::wctpXmlMessage()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlMessage::wctpXmlMessage(
    const wctpXmlMessage &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlMessage::~wctpXmlMessage()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlMessage&
wctpXmlMessage::operator=(
    const wctpXmlMessage &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlMessage::Init(void)
{
    wctpXmlOperation::Init();

    m_MessageType = WCTP_XML_UNDEFINED;
    m_pSubmitRequest = NULL;
    m_pMessageReply = NULL;
    m_pStatusInfo = NULL;
    m_pLookupResponse = NULL;
    m_sequenceNo = "";

    m_title = MESSAGE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;

}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlMessage::Copy(const wctpXmlMessage &rval)
{
    //
    // Copy object type
    //
    wctpXmlOperation::Copy(rval);

    m_MessageType = rval.m_MessageType;

    //
    // Copy pSubmitRequest object if available (maybe holds)
    //

    if(rval.m_pSubmitRequest != NULL)
    {
        m_pSubmitRequest = new wctpXmlSubmitRequest(
            *rval.m_pSubmitRequest);
    }
    else
    {
        m_pSubmitRequest = NULL;
    }

    //
    // Copy pMessageReply object if available (maybe holds)
    //

    if(rval.m_pMessageReply != NULL)
    {
        m_pMessageReply = new wctpXmlMessageReply(
            *rval.m_pMessageReply);
    }
    else
    {
        m_pMessageReply = NULL;
    }

    //
    // Copy pStatusInfo object if available (maybe holds)
    //

    if(rval.m_pStatusInfo != NULL)
    {
        m_pStatusInfo = new wctpXmlStatusInfo(
            *rval.m_pStatusInfo);
    }
    else
    {
        m_pStatusInfo = NULL;
    }

    //
    // Copy pLookupResponse object if available (maybe holds)
    //

    if(rval.m_pLookupResponse != NULL)
    {
        m_pLookupResponse = new wctpXmlLookupResponse(
            *rval.m_pLookupResponse);
    }
    else
    {
        m_pLookupResponse = NULL;
    }

    //
    // Copy member variables
    //

    m_sequenceNo = rval.m_sequenceNo;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlMessage::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pSubmitRequest != NULL) 
    {
        delete m_pSubmitRequest;
        m_pSubmitRequest = NULL;
    }
    if(m_pMessageReply != NULL) 
    {
        delete m_pMessageReply;
        m_pMessageReply = NULL;
    }
    if(m_pStatusInfo != NULL) 
    {
        delete m_pStatusInfo;
        m_pStatusInfo = NULL;
    }
    if(m_pLookupResponse != NULL) 
    {
        delete m_pLookupResponse;
        m_pLookupResponse = NULL;
    }
}



//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlMessage::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-Message");
    dest += ctx.XmlAttribute(
        "sequenceNo",m_sequenceNo);
    dest += ctx.IndentText(">");
    if(m_MessageType == WCTP_XML_SUBMIT_REQUEST && 
       m_pSubmitRequest != NULL)
    {
        dest += m_pSubmitRequest->AssembleOperation(ctx);
    }
    if(m_MessageType == WCTP_XML_MESSAGE_REPLY && 
       m_pMessageReply != NULL)
    {
        dest += m_pMessageReply->AssembleOperation(ctx);
    }
    if(m_MessageType == WCTP_XML_STATUS_INFO && 
       m_pStatusInfo != NULL)
    {
        dest += m_pStatusInfo->AssembleOperation(ctx);
    }
    if(m_MessageType == WCTP_XML_LOOKUP_RESPONSE && 
       m_pLookupResponse != NULL)
    {
        dest += m_pLookupResponse->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-Message>");
    return dest;     
}



//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlMessage::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "sequenceNo")
    {
        m_sequenceNo = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-Message";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlMessage::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    XString command;

    wctp_parse_t res = WCTP_PARSE_ERROR;

    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-Message";
        return WCTP_PARSE_ERROR;
    }
 

    XmlNode *pChild = pTree->children[pos];
    if (pChild->element == "wctp-SubmitRequest")
    {
        HoldSubmitRequest();
        pos++;
        return m_pSubmitRequest->ParseObject(msg,pChild);
    }
    else if (pChild->element == "wctp-MessageReply")
    {
        HoldMessageReply();
        pos++;
        return m_pMessageReply->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-StatusInfo")
    {
        HoldStatusInfo();
        pos++;
        return m_pStatusInfo->ParseObject(msg,pChild);
    }
    else if (pChild->element == "wctp-LookupResponse")
    {
        HoldLookupResponse();
        pos++;
        return m_pLookupResponse->ParseObject(msg,pChild);
    }

    else
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-Message";
        return WCTP_PARSE_ERROR;
    }

    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlMessage::ParseObject(XString& msg, XmlNode *pTree)
{
  
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-Message")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-Message, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-Message does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-Message element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlMessage::GetMessageType(void)
{
    return m_MessageType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to SubmitRequest object
// This function will never return NULL unless new fails...

wctpXmlSubmitRequest*
wctpXmlMessage::GetSubmitRequest(void) 
{
    if(m_pSubmitRequest == NULL)
    {
        m_pSubmitRequest = new wctpXmlSubmitRequest;
    }
    return m_pSubmitRequest;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold SubmitRequest object 

void
wctpXmlMessage::HoldSubmitRequest(void) 
{
    m_MessageType = WCTP_XML_SUBMIT_REQUEST;
    if(m_pSubmitRequest == NULL)
    {
        m_pSubmitRequest = new wctpXmlSubmitRequest;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to MessageReply object
// This function will never return NULL unless new fails...

wctpXmlMessageReply*
wctpXmlMessage::GetMessageReply(void) 
{
    if(m_pMessageReply == NULL)
    {
        m_pMessageReply = new wctpXmlMessageReply;
    }
    return m_pMessageReply;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold MessageReply object 

void
wctpXmlMessage::HoldMessageReply(void) 
{
    m_MessageType = WCTP_XML_MESSAGE_REPLY;
    if(m_pMessageReply == NULL)
    {
        m_pMessageReply = new wctpXmlMessageReply;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to StatusInfo object
// This function will never return NULL unless new fails...

wctpXmlStatusInfo*
wctpXmlMessage::GetStatusInfo(void) 
{
    if(m_pStatusInfo == NULL)
    {
        m_pStatusInfo = new wctpXmlStatusInfo;
    }
    return m_pStatusInfo;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold StatusInfo object 

void
wctpXmlMessage::HoldStatusInfo(void) 
{
    m_MessageType = WCTP_XML_STATUS_INFO;
    if(m_pStatusInfo == NULL)
    {
        m_pStatusInfo = new wctpXmlStatusInfo;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to LookupResponse object
// This function will never return NULL unless new fails...

wctpXmlLookupResponse*
wctpXmlMessage::GetLookupResponse(void) 
{
    if(m_pLookupResponse == NULL)
    {
        m_pLookupResponse = new wctpXmlLookupResponse;
    }
    return m_pLookupResponse;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold LookupResponse object 

void
wctpXmlMessage::HoldLookupResponse(void) 
{
    m_MessageType = WCTP_XML_LOOKUP_RESPONSE;
    if(m_pLookupResponse == NULL)
    {
        m_pLookupResponse = new wctpXmlLookupResponse;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_sequenceNo attribute

const char * 
wctpXmlMessage::GetSequenceNo(void) const 
{
    return m_sequenceNo;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_sequenceNo attribute

void  
wctpXmlMessage::SetSequenceNo(const char * newValue)
{
    m_sequenceNo = newValue;
}

