/* File:  wctpXmlMessageReply.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <stdio.h>

#include "wctpXmlMessageReply.hpp"

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlMessageReply::wctpXmlMessageReply()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlMessageReply::wctpXmlMessageReply(
    const wctpXmlMessageReply &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlMessageReply::~wctpXmlMessageReply()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlMessageReply&
wctpXmlMessageReply::operator=(
    const wctpXmlMessageReply &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlMessageReply::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_MESSAGE_REPLY;
    m_pResponseHeader = new wctpXmlResponseHeader;
    m_pPayload = new wctpXmlPayload;


    m_title = MESSAGE_REPLY_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlMessageReply::Copy(const wctpXmlMessageReply &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy pResponseHeader object if available (should always hold one)
    //

    if(rval.m_pResponseHeader != NULL)
    {
        m_pResponseHeader = new wctpXmlResponseHeader(
            *rval.m_pResponseHeader);
    }
    else
    {
        m_pResponseHeader = new wctpXmlResponseHeader;
    }

    //
    // Copy pPayload object if available (should always hold one)
    //

    if(rval.m_pPayload != NULL)
    {
        m_pPayload = new wctpXmlPayload(
            *rval.m_pPayload);
    }
    else
    {
        m_pPayload = new wctpXmlPayload;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlMessageReply::Free(void)
{
    
    wctpXmlOperation::Free();

    if(m_pResponseHeader != NULL) 
    {
        delete m_pResponseHeader;
        m_pResponseHeader = NULL;
    }
    if(m_pPayload != NULL) 
    {
        delete m_pPayload;
        m_pPayload = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlMessageReply::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlMessageReply::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-MessageReply>");
    if(m_pResponseHeader != NULL)
    {
        dest += m_pResponseHeader->AssembleOperation(ctx);
    }
    if(m_pPayload != NULL)
    {
        dest += m_pPayload->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-MessageReply>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlMessageReply::ParseObject(XString& msg, XmlNode *pTree)
{
    int pos = 0;
 
    //
    // make sure the element matches me
    //
    if(pTree->element != "wctp-MessageReply")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-MessageReply, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-MessageReply does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-MessageReply does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in wctp-ResponseHeader object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pResponseHeader->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-ResponseHeader element expected in ";
        msg += "wctp-MessageReply element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Payload object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pPayload->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Payload element expected in ";
        msg += "wctp-MessageReply element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-MessageReply element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ResponseHeader object
// This function will never return NULL unless new fails...

wctpXmlResponseHeader*
wctpXmlMessageReply::GetResponseHeader(void) 
{
    if(m_pResponseHeader == NULL)
    {
        m_pResponseHeader = new wctpXmlResponseHeader;
    }
    return m_pResponseHeader;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Payload object
// This function will never return NULL unless new fails...

wctpXmlPayload*
wctpXmlMessageReply::GetPayload(void) 
{
    if(m_pPayload == NULL)
    {
        m_pPayload = new wctpXmlPayload;
    }
    return m_pPayload;
}

