/* File:  wctpXmlOperation.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <stdio.h>
#include "wctpXmlOperation.hpp"
#include "XString.hpp"

//Global variables for all requests
XString wctpXmlOperation::g_wctp_version = "wctp-dtd-v1r1";
XString wctpXmlOperation::g_wctp_token   = "";
XString wctpXmlOperation::g_dtd_file     = "http://dtd.wctp.org/wctp-dtd-v1r1.dtd";

//////////////////////////////////
// sets values of static variables used in all requests
// THIS OPERATION IS NOT THREAD-SAFE: CALL IT BEFORE STARTING
// YOUR THREADS. IF YOU CALL IT DURING THREAD EXECUTION, YOU
// MAY GET INCONSISTENT HEADERS/ATTRIBUTES ON REQUESTS THAT 
// ARE IN PROOGRESS AT THE TIME OF THE CHANGE.
// Returns nothing
void
wctpXmlOperation::SetDefaultRequestValues(const char* wctp_version,
                                          const char* dtd_file,
                                          const char* wctp_token)
{
    if (wctp_version != NULL)
    {   // have a version default passed in, keep it for future use
        g_wctp_version = wctp_version;
    }

    if (dtd_file != NULL)
    {   // have a dtd file default passed in, keep it for future use
        g_dtd_file = dtd_file;
    }

    if (wctp_token != NULL)
    {   // have a token default passed in, keep it for future use
        g_wctp_token = wctp_token;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlOperation::wctpXmlOperation()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlOperation::wctpXmlOperation(
    const wctpXmlOperation &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlOperation::~wctpXmlOperation()
{


    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlOperation&
wctpXmlOperation::operator=(
    const wctpXmlOperation &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlOperation::Init(void)
{
    m_OperationType = WCTP_XML_UNDEFINED;

    m_xmlPrefix     = WCTP_PREFIX;
    m_xmlVersion    = g_wctp_version;
    m_xmlToken      = g_wctp_token;
    m_dtdFile       = g_dtd_file;

    m_title         = OPERATION_STRING;
    m_xmlOperation  = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlOperation::Copy(const wctpXmlOperation &rval)
{
    //
    // Copy object type
    //

    m_OperationType = rval.m_OperationType;

    m_xmlOperation  = rval.m_xmlOperation;
    m_xmlPrefix     = rval.m_xmlPrefix;
    m_xmlVersion    = rval.m_xmlVersion;
    m_xmlToken      = rval.m_xmlToken;
    m_dtdFile       = rval.m_dtdFile;
    m_title         = rval.m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlOperation::Free(void)
{
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlOperation::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object 

const char *
wctpXmlOperation::AssembleRequestString(void)
{
    return Assemble();
}
//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object 

const char *
wctpXmlOperation::Assemble(void)
{
    XmlContext ctx;
    m_xml = "";
    m_xml += "<?xml version=\"1.0\"?>\n";


    m_xml += AssembleOperationHeader(ctx);
    m_xml += AssembleOperation(ctx);

    m_xml += AssembleOperationFooter(ctx);

    return m_xml.c_str();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlOperation::AssembleOperationHeader(XmlContext& ctx)
{
    XString dest;
    
    dest = "<!DOCTYPE ";
    dest += m_xmlPrefix;
    dest += "-Operation SYSTEM ";
    dest += "\"";
    dest += m_dtdFile;
    dest +="\">\n";

    XString temp = "<";
    temp += m_xmlPrefix;
    temp += "-Operation";
    
    dest += ctx.OpenIndent(temp );
                
    XString attrib = m_xmlPrefix;
    attrib += "Version";
    dest += ctx.XmlAttribute(attrib, m_xmlVersion);

    attrib = m_xmlPrefix;
    attrib += "Token";
    dest += ctx.XmlAttribute(attrib, m_xmlToken);

    dest += ctx.CloseIndent(">" );

    return dest;
}


XString 
wctpXmlOperation::AssembleOperation(XmlContext& ctx)
{

    XString dest = "";
    return dest;

}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlOperation::AssembleOperationFooter(XmlContext& ctx)
{
    XString dest;
    XString temp = "</";
    temp += m_xmlPrefix;
    temp += "-Operation>";

    dest += ctx.CloseIndent(temp);
    
    return dest;
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlOperation::Assemble(XmlContext& ctx)
{
    XString dest;
    return dest;     
}

/////    /////////////////////////////////////////////////////////////////////////
// Get last parse message

const char *
wctpXmlOperation::LastParseMessage()
{
    return m_parse_msg.c_str();
}

//////////////////////////////////////////////////////////////////////////////
// Parse given xml text

wctp_parse_t
wctpXmlOperation::Parse(const char * xml)
{

    XML_Parser p;


    XmlNode *pTree = XmlNode::parse(m_parse_msg, xml, p);
    if(pTree == NULL)
    {
        return WCTP_PARSE_ERROR;
    }

    wctp_parse_t res = ParseHeader(m_parse_msg, pTree);

    delete pTree;
    XML_ParserFree(p);
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlOperation::ParseHeaderAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if (key == "wctpVersion")
    {
        m_xmlVersion = val;
    }
    else if (key == "wctpToken")
    {
        m_xmlToken = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element ";
        msg += m_xmlOperation;
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlOperation::ParseObject( XString& msg, XmlNode *pTree)
{
    wctp_parse_t result = WCTP_PARSE_OKAY;


    XString operation = pTree->element;

    if (operation == "wctp-SubmitRequest")
    {
        m_OperationType = WCTP_XML_SUBMIT_REQUEST;
    }
    else if (operation == "wctp-MessageReply")
    {
        m_OperationType = WCTP_XML_MESSAGE_REPLY;
    }    
    else if (operation == "wctp-StatusInfo")
    {
        m_OperationType = WCTP_XML_STATUS_INFO;
    }    
    else if (operation == "wctp-LookupResponse")
    {
        m_OperationType = WCTP_XML_LOOKUP_RESPONSE;
    }
    else 
    {
        m_OperationType = WCTP_XML_UNDEFINED;
        result = WCTP_PARSE_ERROR;
    }


    return result;
}

/////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlOperation::ParseHeader(
    XString& msg,
    XmlNode *pTree)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;

    //
    // make sure the element matches me
    //
    XString OperationString = m_xmlPrefix;
    OperationString += "-Operation";
    
    if (OperationString != pTree->element)
    {
        msg = "element mismatch, expected ";
           msg += OperationString;
        msg += ", but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseHeaderAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = m_xmlOperation;
        msg += " does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

       
    // Load in child objects
    //
    int pos = 0;

    //
    // Load in an object from a set
    //
    XmlNode *pChild = pTree->children[pos];

    res = ParseObject(msg, pChild);

    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }


    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor for Title

XString& 
wctpXmlOperation::GetTitle()
{
    return m_title;
}
//////////////////////////////////////////////////////////////////////////////
// Accessor for Title

void 
wctpXmlOperation::SetTitle(const char * title)
{
    m_title = title;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlOperation::GetOperationType(void)
{
    return m_OperationType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_dtdFile attribute

const char * 
wctpXmlOperation::GetDTDFile(void) const 
{
    return m_dtdFile;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_dtdFile attribute

void  
wctpXmlOperation::SetDTDFile(const char * newValue)
{
    m_dtdFile = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_wctpVersion attribute

const char * 
wctpXmlOperation::GetVersion(void) const 
{
    return m_xmlVersion;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_wctpVersion attribute

void  
wctpXmlOperation::SetVersion(const char * newValue)
{
    m_xmlVersion = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_wctpToken attribute

const char * 
wctpXmlOperation::GetToken(void) const 
{
    return m_xmlToken;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_wctpToken attribute

void  
wctpXmlOperation::SetToken(const char * newValue)
{
    m_xmlToken = newValue;
}

////////////////////////////
// Set Prefix - used for any other libraries that may want to use the Payload object 
void 
wctpXmlOperation::SetPrefix(const char * newValue)
{
    m_xmlPrefix = newValue;
    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}
