/* File:  wctpXmlOriginator.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlOriginator.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlOriginator::wctpXmlOriginator()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlOriginator::wctpXmlOriginator(
    const wctpXmlOriginator &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlOriginator::~wctpXmlOriginator()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlOriginator&
wctpXmlOriginator::operator=(
    const wctpXmlOriginator &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlOriginator::Init(void)
{
    wctpXmlOperation::Init();

    m_miscInfo = "";
    m_securityCode = "";
    m_senderID = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlOriginator::Copy(const wctpXmlOriginator &rval)
{
    //
    // Copy member variables
    //
    wctpXmlOperation::Copy(rval);

    m_miscInfo = rval.m_miscInfo;
    m_securityCode = rval.m_securityCode;
    m_senderID = rval.m_senderID;
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlOriginator::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-Originator");
    dest += ctx.XmlAttribute(
        "miscInfo",m_miscInfo);
    dest += ctx.XmlAttribute(
        "securityCode",m_securityCode);
    dest += ctx.XmlAttribute(
        "senderID",m_senderID);
    dest += ctx.CloseIndent("/>");
    return dest;     
}



//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlOriginator::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "miscInfo")
    {
        m_miscInfo = val;
    }
    else if(key == "securityCode")
    {
        m_securityCode = val;
    }
    else if(key == "senderID")
    {
        m_senderID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-Originator";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlOriginator::ParseObject(XString& msg,XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-Originator")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-Originator, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-Originator does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-Originator element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_miscInfo attribute

const char * 
wctpXmlOriginator::GetMiscInfo(void) const 
{
    return m_miscInfo;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_miscInfo attribute

void  
wctpXmlOriginator::SetMiscInfo(const char * newValue)
{
    m_miscInfo = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_securityCode attribute

const char * 
wctpXmlOriginator::GetSecurityCode(void) const 
{
    return m_securityCode;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_securityCode attribute

void  
wctpXmlOriginator::SetSecurityCode(const char * newValue)
{
    m_securityCode = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_senderID attribute

const char * 
wctpXmlOriginator::GetSenderID(void) const 
{
    return m_senderID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_senderID attribute

void  
wctpXmlOriginator::SetSenderID(const char * newValue)
{
    m_senderID = newValue;
}

