/* File:  wctpXmlPollResponse.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlPollResponse.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlPollResponse::wctpXmlPollResponse()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlPollResponse::wctpXmlPollResponse(
    const wctpXmlPollResponse &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlPollResponse::~wctpXmlPollResponse()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlPollResponse&
wctpXmlPollResponse::operator=(
    const wctpXmlPollResponse &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlPollResponse::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_POLL_RESPONSE;

    m_PollResponseType = WCTP_XML_UNDEFINED;
    m_pNoMessages = NULL;
    m_minNextPollInterval = "";

    m_title = POLL_RESPONSE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;


}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlPollResponse::Copy(const wctpXmlPollResponse &rval)
{

    wctpXmlOperation::Copy(rval);
    //
    // Copy object type
    //

    m_PollResponseType = rval.m_PollResponseType;

    //
    // Copy pMessage object(s) if available (holds one or more)
    //

    if(rval.m_vMessage.size() > 0)
    {
        int i;
        for(i=0; i<rval.m_vMessage.size(); i++)
        {
            wctpXmlMessage *pMessage = 
                new wctpXmlMessage(
                *(rval.m_vMessage[i]));
            m_vMessage.push(pMessage);
        }
    }

    //
    // Copy pNoMessages object if available (maybe holds)
    //

    if(rval.m_pNoMessages != NULL)
    {
        m_pNoMessages = new wctpXmlNoMessages(
            *rval.m_pNoMessages);
    }
    else
    {
        m_pNoMessages = NULL;
    }

    //
    // Copy member variables
    //

    m_minNextPollInterval = rval.m_minNextPollInterval;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlPollResponse::Free(void)
{
    wctpXmlOperation::Free();

    m_vMessage.destroy();
    if(m_pNoMessages != NULL) 
    {
        delete m_pNoMessages;
        m_pNoMessages = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlPollResponse::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct an XML string for Message object(s)

XString
wctpXmlPollResponse::AssembleMessage(XmlContext& ctx)
{
    XString dest;
    int i;
    for(i=0; i<m_vMessage.size(); i++) 
    {
        dest += m_vMessage[i]->AssembleOperation(ctx); 
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlPollResponse::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-PollResponse");
    dest += ctx.XmlAttribute(
        "minNextPollInterval",m_minNextPollInterval);
    dest += ctx.IndentText(">");
    if(m_PollResponseType == WCTP_XML_MESSAGE)
    {
        dest += AssembleMessage(ctx);
    }
    if(m_PollResponseType == WCTP_XML_NO_MESSAGES && 
       m_pNoMessages != NULL)
    {
        dest += m_pNoMessages->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-PollResponse>");
    return dest;     
}



//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlPollResponse::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "minNextPollInterval")
    {
        m_minNextPollInterval = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-PollResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse one or more Message object(s)

wctp_parse_t
wctpXmlPollResponse::ParseMessage(
    XString& msg,
    XmlNode *pTree,
    int& pos)
{
    bool flag = false;
    while(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if(pChild->element != "wctp-Message")
        {
            break;
        }
        flag = true;
        pos++;
        wctpXmlMessage *pMessage = AddMessage();
        wctp_parse_t res = pMessage->ParseObject(msg, pChild); 
        if(res != WCTP_PARSE_OKAY)
        {
            return res; 
        }
    }
    if(!flag) 
    {
        msg = "expect at least one wctp-Message ";
        msg += "inside wctp-PollResponse";
        return WCTP_PARSE_ERROR;        
    }
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlPollResponse::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-PollResponse";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-Message")
    {
        HoldMessage();
        res = ParseMessage(msg,pTree,pos);
    }
    else if(pChild->element == "wctp-NoMessages")
    {
        HoldNoMessages();
        pos++;
        res = m_pNoMessages->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-PollResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlPollResponse::ParseObject(XString& msg, XmlNode *pTree)
{
    int pos = 0;

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-PollResponse")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-PollResponse, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-PollResponse does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-PollResponse element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlPollResponse::GetPollResponseType(void)
{
    return m_PollResponseType;
}

//////////////////////////////////////////////////////////////////////////////
// Clear all Message elements

void
wctpXmlPollResponse::ClearMessage(void) 
{
    m_vMessage.destroy();
}

//////////////////////////////////////////////////////////////////////////////
// Get Message element at given position

wctpXmlMessage *
wctpXmlPollResponse::GetMessage2(int pos)
{
    if(pos >= 0 && pos < m_vMessage.size())
    {
        return m_vMessage[pos];
    }
    return NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Add Message element to end of array

wctpXmlMessage *
wctpXmlPollResponse::AddMessage(void)
{
    wctpXmlMessage *pMessage = 
        new wctpXmlMessage;
    m_vMessage.push(pMessage);
    return pMessage;
}

//////////////////////////////////////////////////////////////////////////////
// Erase Message element at given position

void
wctpXmlPollResponse::EraseMessage(int pos)
{
    wctpXmlMessage *pMessage = 
        m_vMessage.remove(pos);
    delete pMessage;
    pMessage = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Return number of Message elements

int
wctpXmlPollResponse::NumMessage(void)
{
    return m_vMessage.size();
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Message object 

void
wctpXmlPollResponse::HoldMessage(void)
{
    m_PollResponseType = WCTP_XML_MESSAGE;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to NoMessages object
// This function will never return NULL unless new fails...

wctpXmlNoMessages*
wctpXmlPollResponse::GetNoMessages(void) 
{
    if(m_pNoMessages == NULL)
    {
        m_pNoMessages = new wctpXmlNoMessages;
    }
    return m_pNoMessages;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold NoMessages object 

void
wctpXmlPollResponse::HoldNoMessages(void) 
{
    m_PollResponseType = WCTP_XML_NO_MESSAGES;
    if(m_pNoMessages == NULL)
    {
        m_pNoMessages = new wctpXmlNoMessages;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_minNextPollInterval attribute

const char * 
wctpXmlPollResponse::GetMinNextPollInterval(void) const 
{
    return m_minNextPollInterval;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_minNextPollInterval attribute

void  
wctpXmlPollResponse::SetMinNextPollInterval(const char * newValue)
{
    m_minNextPollInterval = newValue;
}

