/* File:  wctpXmlSubmitClientHeader.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlSubmitClientHeader.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlSubmitClientHeader::wctpXmlSubmitClientHeader()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlSubmitClientHeader::wctpXmlSubmitClientHeader(
    const wctpXmlSubmitClientHeader &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlSubmitClientHeader::~wctpXmlSubmitClientHeader()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlSubmitClientHeader&
wctpXmlSubmitClientHeader::operator=(
    const wctpXmlSubmitClientHeader &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlSubmitClientHeader::Init(void)
{
    wctpXmlOperation::Init();

    m_pClientOriginator = new wctpXmlClientOriginator;
    m_use_ClientMessageControl = true;
    m_pClientMessageControl = NULL;
    m_pRecipient = new wctpXmlRecipient;
    m_submitTimestamp = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlSubmitClientHeader::Copy(const wctpXmlSubmitClientHeader &rval)
{

    wctpXmlOperation::Copy(rval);

    //
    // Copy pClientOriginator object if available (should always hold one)
    //

    if(rval.m_pClientOriginator != NULL)
    {
        m_pClientOriginator = new wctpXmlClientOriginator(
            *rval.m_pClientOriginator);
    }
    else
    {
        m_pClientOriginator = new wctpXmlClientOriginator;
    }

    //
    // Copy pClientMessageControl object if available (holds zero or one)
    //

    if(rval.m_pClientMessageControl != NULL)
    {
        m_use_ClientMessageControl = rval.m_use_ClientMessageControl;
        m_pClientMessageControl = new wctpXmlClientMessageControl(
            *rval.m_pClientMessageControl);
    }
    else
    {
        m_use_ClientMessageControl = false;
        m_pClientMessageControl = NULL;
    }

    //
    // Copy pRecipient object if available (should always hold one)
    //

    if(rval.m_pRecipient != NULL)
    {
        m_pRecipient = new wctpXmlRecipient(
            *rval.m_pRecipient);
    }
    else
    {
        m_pRecipient = new wctpXmlRecipient;
    }

    //
    // Copy member variables
    //

    m_submitTimestamp = rval.m_submitTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlSubmitClientHeader::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pClientOriginator != NULL) 
    {
        delete m_pClientOriginator;
        m_pClientOriginator = NULL;
    }
    if(m_pClientMessageControl != NULL) 
    {
        delete m_pClientMessageControl;
        m_pClientMessageControl = NULL;
    }
    if(m_pRecipient != NULL) 
    {
        delete m_pRecipient;
        m_pRecipient = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlSubmitClientHeader::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-SubmitClientHeader");
    dest += ctx.XmlAttribute(
        "submitTimestamp",m_submitTimestamp);
    dest += ctx.IndentText(">");
    if(m_pClientOriginator != NULL)
    {
        dest += m_pClientOriginator->AssembleOperation(ctx);
    }
    if(m_pClientMessageControl != NULL && m_use_ClientMessageControl)
    {
        dest += m_pClientMessageControl->AssembleOperation(ctx);
    }
    if(m_pRecipient != NULL)
    {
        dest += m_pRecipient->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-SubmitClientHeader>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlSubmitClientHeader::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "submitTimestamp")
    {
        m_submitTimestamp = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-SubmitClientHeader";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlSubmitClientHeader::ParseObject(XString& msg, XmlNode *pTree)
{
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-SubmitClientHeader")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-SubmitClientHeader, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-SubmitClientHeader does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-ClientOriginator object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pClientOriginator->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-ClientOriginator element expected in ";
        msg += "wctp-SubmitClientHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in zero or one ClientMessageControl objects
    //

    DontUseClientMessageControl(); 
    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if(pChild->element == "wctp-ClientMessageControl")
        {
            pos++;
            UseClientMessageControl();
            res = m_pClientMessageControl->ParseObject(msg,pChild);
            if(res != WCTP_PARSE_OKAY)
            {
                return WCTP_PARSE_ERROR;
            }
        }
    }

    //
    // Load in wctp-Recipient object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pRecipient->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Recipient element expected in ";
        msg += "wctp-SubmitClientHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-SubmitClientHeader element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientOriginator object
// This function will never return NULL unless new fails...

wctpXmlClientOriginator*
wctpXmlSubmitClientHeader::GetClientOriginator(void) 
{
    if(m_pClientOriginator == NULL)
    {
        m_pClientOriginator = new wctpXmlClientOriginator;
    }
    return m_pClientOriginator;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientMessageControl object
// This function will never return NULL unless new fails...

wctpXmlClientMessageControl*
wctpXmlSubmitClientHeader::GetClientMessageControl(void) 
{
    if(m_pClientMessageControl == NULL)
    {
        m_pClientMessageControl = new wctpXmlClientMessageControl;
    }
    return m_pClientMessageControl;
}

//////////////////////////////////////////////////////////////////////////////
// Specify to use ClientMessageControl object 

void
wctpXmlSubmitClientHeader::UseClientMessageControl(void) 
{
    m_use_ClientMessageControl = true;
    if(m_pClientMessageControl == NULL)
    {
        m_pClientMessageControl = new wctpXmlClientMessageControl;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Specify not to use ClientMessageControl object 

void
wctpXmlSubmitClientHeader::DontUseClientMessageControl(void) 
{
    m_use_ClientMessageControl = false;
}

//////////////////////////////////////////////////////////////////////////////
// Find out if using ClientMessageControl object 

bool
wctpXmlSubmitClientHeader::IsUsingClientMessageControl(void) 
{
    return m_use_ClientMessageControl;
}

//////////////////////////////////////////////////////////////////////////////
// Clear ClientMessageControl object 

void
wctpXmlSubmitClientHeader::ClearClientMessageControl(void) 
{
    m_use_ClientMessageControl = false;
    if(m_pClientMessageControl != NULL)
    {
        delete m_pClientMessageControl;
        m_pClientMessageControl = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Recipient object
// This function will never return NULL unless new fails...

wctpXmlRecipient*
wctpXmlSubmitClientHeader::GetRecipient(void) 
{
    if(m_pRecipient == NULL)
    {
        m_pRecipient = new wctpXmlRecipient;
    }
    return m_pRecipient;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_submitTimestamp attribute

const char * 
wctpXmlSubmitClientHeader::GetSubmitTimestamp(void) const 
{
    return m_submitTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_submitTimestamp attribute

void  
wctpXmlSubmitClientHeader::SetSubmitTimestamp(const char * newValue)
{
    m_submitTimestamp = newValue;
}

