/* File:  wctpXmlSubmitClientMessage.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlSubmitClientMessage.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlSubmitClientMessage::wctpXmlSubmitClientMessage()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlSubmitClientMessage::wctpXmlSubmitClientMessage(
    const wctpXmlSubmitClientMessage &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlSubmitClientMessage::~wctpXmlSubmitClientMessage()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlSubmitClientMessage&
wctpXmlSubmitClientMessage::operator=(
    const wctpXmlSubmitClientMessage &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlSubmitClientMessage::Init(void)
{
    
    wctpXmlOperation::Init();
    
    m_OperationType = WCTP_XML_SUBMIT_CLIENT_MESSAGE;
    
    m_pSubmitClientHeader = new wctpXmlSubmitClientHeader;
    m_pPayload = new wctpXmlPayload;
    
    m_title = SUBMIT_CLIENT_MESSAGE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;



}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlSubmitClientMessage::Copy(const wctpXmlSubmitClientMessage &rval)
{
    wctpXmlOperation::Copy(rval);
    //
    // Copy pSubmitClientHeader object if available (should always hold one)
    //

    if(rval.m_pSubmitClientHeader != NULL)
    {
        m_pSubmitClientHeader = new wctpXmlSubmitClientHeader(
            *rval.m_pSubmitClientHeader);
    }
    else
    {
        m_pSubmitClientHeader = new wctpXmlSubmitClientHeader;
    }

    //
    // Copy pPayload object if available (should always hold one)
    //

    if(rval.m_pPayload != NULL)
    {
        m_pPayload = new wctpXmlPayload(
            *rval.m_pPayload);
    }
    else
    {
        m_pPayload = new wctpXmlPayload;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlSubmitClientMessage::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pSubmitClientHeader != NULL) 
    {
        delete m_pSubmitClientHeader;
        m_pSubmitClientHeader = NULL;
    }
    if(m_pPayload != NULL) 
    {
        delete m_pPayload;
        m_pPayload = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlSubmitClientMessage::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlSubmitClientMessage::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-SubmitClientMessage>");
    if(m_pSubmitClientHeader != NULL)
    {
        dest += m_pSubmitClientHeader->AssembleOperation(ctx);
    }
    if(m_pPayload != NULL)
    {
        dest += m_pPayload->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-SubmitClientMessage>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlSubmitClientMessage::ParseObject(XString& msg,XmlNode *pTree )
{
    int pos = 0;

    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-SubmitClientMessage")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-SubmitClientMessage, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-SubmitClientMessage does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-SubmitClientMessage does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //
    wctp_parse_t res;

    //
    // Load in wctp-SubmitClientHeader object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pSubmitClientHeader->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-SubmitClientHeader element expected in ";
        msg += "wctp-SubmitClientMessage element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Payload object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pPayload->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Payload element expected in ";
        msg += "wctp-SubmitClientMessage element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-SubmitClientMessage element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to SubmitClientHeader object
// This function will never return NULL unless new fails...

wctpXmlSubmitClientHeader*
wctpXmlSubmitClientMessage::GetSubmitClientHeader(void) 
{
    if(m_pSubmitClientHeader == NULL)
    {
        m_pSubmitClientHeader = new wctpXmlSubmitClientHeader;
    }
    return m_pSubmitClientHeader;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Payload object
// This function will never return NULL unless new fails...

wctpXmlPayload*
wctpXmlSubmitClientMessage::GetPayload(void) 
{
    if(m_pPayload == NULL)
    {
        m_pPayload = new wctpXmlPayload;
    }
    return m_pPayload;
}


