/* File:  wctpXmlSubmitRequest.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlSubmitRequest.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlSubmitRequest::wctpXmlSubmitRequest()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlSubmitRequest::wctpXmlSubmitRequest(
    const wctpXmlSubmitRequest &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlSubmitRequest::~wctpXmlSubmitRequest()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlSubmitRequest&
wctpXmlSubmitRequest::operator=(
    const wctpXmlSubmitRequest &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlSubmitRequest::Init(void)
{
    wctpXmlOperation::Init();

	m_OperationType = WCTP_XML_SUBMIT_REQUEST;

    m_pSubmitHeader = new wctpXmlSubmitHeader;
    m_pPayload = new wctpXmlPayload;

    m_title = SUBMIT_REQUEST_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;


}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlSubmitRequest::Copy(const wctpXmlSubmitRequest &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy pSubmitHeader object if available (should always hold one)
    //

    if(rval.m_pSubmitHeader != NULL)
    {
        m_pSubmitHeader = new wctpXmlSubmitHeader(
            *rval.m_pSubmitHeader);
    }
    else
    {
        m_pSubmitHeader = new wctpXmlSubmitHeader;
    }

    //
    // Copy pPayload object if available (should always hold one)
    //

    if(rval.m_pPayload != NULL)
    {
        m_pPayload = new wctpXmlPayload(
            *rval.m_pPayload);
    }
    else
    {
        m_pPayload = new wctpXmlPayload;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlSubmitRequest::Free(void)
{

    wctpXmlOperation::Free();

    if(m_pSubmitHeader != NULL) 
    {
        delete m_pSubmitHeader;
        m_pSubmitHeader = NULL;
    }
    if(m_pPayload != NULL) 
    {
        delete m_pPayload;
        m_pPayload = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlSubmitRequest::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlSubmitRequest::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-SubmitRequest>");
    if(m_pSubmitHeader != NULL)
    {
        dest += m_pSubmitHeader->AssembleOperation(ctx);
    }
    if(m_pPayload != NULL)
    {
        dest += m_pPayload->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-SubmitRequest>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlSubmitRequest::ParseObject(XString& msg,XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-SubmitRequest")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-SubmitRequest, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-SubmitRequest does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-SubmitRequest does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-SubmitHeader object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pSubmitHeader->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-SubmitHeader element expected in ";
        msg += "wctp-SubmitRequest element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Payload object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pPayload->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Payload element expected in ";
        msg += "wctp-SubmitRequest element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-SubmitRequest element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to SubmitHeader object
// This function will never return NULL unless new fails...

wctpXmlSubmitHeader*
wctpXmlSubmitRequest::GetSubmitHeader(void) 
{
    if(m_pSubmitHeader == NULL)
    {
        m_pSubmitHeader = new wctpXmlSubmitHeader;
    }
    return m_pSubmitHeader;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Payload object
// This function will never return NULL unless new fails...

wctpXmlPayload*
wctpXmlSubmitRequest::GetPayload(void) 
{
    if(m_pPayload == NULL)
    {
        m_pPayload = new wctpXmlPayload;
    }
    return m_pPayload;
}

