/* File:  wctpXmlSuccess.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <stdio.h>
#include "wctpXmlSuccess.hpp"

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlSuccess::wctpXmlSuccess()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlSuccess::wctpXmlSuccess(
    const wctpXmlSuccess &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlSuccess::~wctpXmlSuccess()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlSuccess&
wctpXmlSuccess::operator=(
    const wctpXmlSuccess &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlSuccess::Init(void)
{
    wctpXmlOperation::Init();

    m_pcdata = "";
    m_successCode = "";
    m_successText = "";

    m_title = SUCCESS_STRING;
    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;


}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlSuccess::Copy(const wctpXmlSuccess &rval)
{
    //
    // Copy member variables
    //

    wctpXmlOperation::Copy(rval);

    m_pcdata = rval.m_pcdata;
    m_successCode = rval.m_successCode;
    m_successText = rval.m_successText;
    

}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlSuccess::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    XString temp;

    temp = "<";
    temp += m_xmlOperation;
    dest += ctx.OpenIndent(temp);

    dest += ctx.XmlAttribute(
        "successCode",m_successCode);
    dest += ctx.XmlAttribute(
        "successText",m_successText);
    dest += ctx.IndentText(">");
    dest += ctx.XmlPcdata(m_pcdata);

    temp = "</";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.CloseIndent(temp);

    
    return dest;     
}
//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlSuccess::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "successCode")
    {
        m_successCode = val;
    }
    else if(key == "successText")
    {
        m_successText = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element ";
        msg += m_xmlOperation;
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlSuccess::ParseObject (
    XString& msg,
    XmlNode *pTree)
{

    //
    // make sure the element matches me
    //
    if (pTree->element != m_xmlOperation)
    {
        msg = "element mismatch, expected ";
        msg += m_xmlOperation;
        msg += ", but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

    //
    // collect pcdata 
    //

    m_pcdata = pTree->pcdata.both_trim();

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = m_xmlOperation;
        msg += " element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_pcdata attribute

const char * 
wctpXmlSuccess::GetPcdata(void) const 
{
    return m_pcdata;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_pcdata attribute

void  
wctpXmlSuccess::SetPcdata(const char * newValue)
{
    m_pcdata = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_successCode attribute

const char * 
wctpXmlSuccess::GetSuccessCode(void) const 
{
    return m_successCode;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_successCode attribute

void  
wctpXmlSuccess::SetSuccessCode(const char * newValue)
{
    m_successCode = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_successText attribute

const char * 
wctpXmlSuccess::GetSuccessText(void) const 
{
    return m_successText;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_successText attribute

void  
wctpXmlSuccess::SetSuccessText(const char * newValue)
{
    m_successText = newValue;
}

