/* File:  wctpXmlVersionContactInfo.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlVersionContactInfo.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlVersionContactInfo::wctpXmlVersionContactInfo()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlVersionContactInfo::wctpXmlVersionContactInfo(
    const wctpXmlVersionContactInfo &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlVersionContactInfo::~wctpXmlVersionContactInfo()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlVersionContactInfo&
wctpXmlVersionContactInfo::operator=(
    const wctpXmlVersionContactInfo &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlVersionContactInfo::Init(void)
{
    wctpXmlOperation::Init();

    m_email = "";
    m_phone = "";
    m_www = "";
    m_info = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlVersionContactInfo::Copy(const wctpXmlVersionContactInfo &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy member variables
    //

    m_email = rval.m_email;
    m_phone = rval.m_phone;
    m_www = rval.m_www;
    m_info = rval.m_info;
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlVersionContactInfo::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ContactInfo");
    dest += ctx.XmlAttribute(
        "email",m_email);
    dest += ctx.XmlAttribute(
        "phone",m_phone);
    dest += ctx.XmlAttribute(
        "www",m_www);
    dest += ctx.XmlAttribute(
        "info",m_info);
    dest += ctx.IndentText(">");
    dest += ctx.CloseIndent("</wctp-ContactInfo>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlVersionContactInfo::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "email")
    {
        m_email = val;
    }
    else if(key == "phone")
    {
        m_phone = val;
    }
    else if(key == "www")
    {
        m_www = val;
    }
    else if(key == "info")
    {
        m_info = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-ContactInfo";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlVersionContactInfo::ParseObject(XString& msg, XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ContactInfo")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ContactInfo, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ContactInfo does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //
    if(pTree->children.size() > 0)
    {
        msg = "wctp-ContactInfo element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_email attribute

const char * 
wctpXmlVersionContactInfo::GetEmail(void) const 
{
    return m_email;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_email attribute

void  
wctpXmlVersionContactInfo::SetEmail(const char * newValue)
{
    m_email = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_phone attribute

const char * 
wctpXmlVersionContactInfo::GetPhone(void) const 
{
    return m_phone;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_phone attribute

void  
wctpXmlVersionContactInfo::SetPhone(const char * newValue)
{
    m_phone = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_www attribute

const char * 
wctpXmlVersionContactInfo::GetWWW(void) const 
{
    return m_www;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_www attribute

void  
wctpXmlVersionContactInfo::SetWWW(const char * newValue)
{
    m_www = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_info attribute

const char * 
wctpXmlVersionContactInfo::GetInfo(void) const 
{
    return m_info;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_info attribute

void  
wctpXmlVersionContactInfo::SetInfo(const char * newValue)
{
    m_info = newValue;
}
