/* File:  wctpXmlVersionDTDSupport.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlVersionDTDSupport.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlVersionDTDSupport::wctpXmlVersionDTDSupport()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlVersionDTDSupport::wctpXmlVersionDTDSupport(
    const wctpXmlVersionDTDSupport &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlVersionDTDSupport::~wctpXmlVersionDTDSupport()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlVersionDTDSupport&
wctpXmlVersionDTDSupport::operator=(
    const wctpXmlVersionDTDSupport &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlVersionDTDSupport::Init(void)
{
    wctpXmlOperation::Init();

    m_dtdName = "";
    m_supportType = "";
    m_exceptions = "";
    m_supportUntil = "";
    m_replacement = "";
    m_verToken = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlVersionDTDSupport::Copy(const wctpXmlVersionDTDSupport &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy member variables
    //

    m_dtdName = rval.m_dtdName;
    m_supportType = rval.m_supportType;
    m_exceptions = rval.m_exceptions;
    m_supportUntil = rval.m_supportUntil;
    m_replacement = rval.m_replacement;
    m_verToken = rval.m_verToken;
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlVersionDTDSupport::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-DTDSupport");
    dest += ctx.XmlAttribute(
        "dtdName",m_dtdName);
    dest += ctx.XmlAttribute(
        "supportType",m_supportType);
    dest += ctx.XmlAttribute(
        "exceptions",m_exceptions);
    dest += ctx.XmlAttribute(
        "supportUntil",m_supportUntil);
    dest += ctx.XmlAttribute(
        "replacement",m_replacement);
    dest += ctx.XmlAttribute(
        "verToken",m_verToken);
    dest += ctx.IndentText(">");
    dest += ctx.CloseIndent("</wctp-DTDSupport>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlVersionDTDSupport::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "dtdName")
    {
        m_dtdName = val;
    }
    else if(key == "supportType")
    {
        m_supportType = val;
    }
    else if(key == "exceptions")
    {
        m_exceptions = val;
    }
    else if(key == "supportUntil")
    {
        m_supportUntil = val;

    }
    else if(key == "replacement")
    {
        m_replacement = val;
    }
    else if(key == "verToken")
    {
        m_verToken = val;

    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-DTDSupport";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlVersionDTDSupport::ParseObject(XString& msg, XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-DTDSupport")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-DTDSupport, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //
     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-DTDSupport does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //
    if(pTree->children.size() > 0)
    {
        msg = "wctp-DTDSupport element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_dtdName attribute

const char * 
wctpXmlVersionDTDSupport::GetDTDName(void) const 
{
    return m_dtdName;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_dtdName attribute

void  
wctpXmlVersionDTDSupport::SetDTDName(const char * newValue)
{
    m_dtdName = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_supportType attribute

const char * 
wctpXmlVersionDTDSupport::GetSupportType(void) const 
{
    return m_supportType;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_supportType attribute

void  
wctpXmlVersionDTDSupport::SetSupportType(const char * newValue)
{
    m_supportType = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_exceptions attribute

const char * 
wctpXmlVersionDTDSupport::GetExceptions(void) const 
{
    return m_exceptions;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_exceptions attribute

void  
wctpXmlVersionDTDSupport::SetExceptions(const char * newValue)
{
    m_exceptions = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_supportUntil attribute

const char * 
wctpXmlVersionDTDSupport::GetSupportUntil(void) const 
{
    return m_supportUntil;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_supportUntil attribute

void  
wctpXmlVersionDTDSupport::SetSupportUntil(const char * newValue)
{
    m_supportUntil = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_replacement attribute

const char * 
wctpXmlVersionDTDSupport::GetReplacement(void) const 
{
    return m_replacement;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_replacement attribute

void  
wctpXmlVersionDTDSupport::SetReplacement(const char * newValue)
{
    m_replacement = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_verToken attribute

const char * 
wctpXmlVersionDTDSupport::GetVerToken(void) const 
{
    return m_verToken;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_verToken attribute

void  
wctpXmlVersionDTDSupport::SetVerToken(const char * newValue)
{
    m_verToken = newValue;
}
